@@ -11,6 +11,7 @@ from ..device.basedevice import (
     PLOT_SETTING,
     PLOT_START,
 )
+from .parameters import Parameters
 
 """
 
@@ -37,9 +38,18 @@ of zero will remain zero.
 """
 
 
-class PlotPlanner:
-    def __init__(self, settings):
-        self.settings = settings
+class PlotPlanner(Parameters):
+    def __init__(
+        self,
+        settings,
+        single=True,
+        smooth=True,
+        ppi=True,
+        shift=True,
+        group=True,
+        **kwargs,
+    ):
+        super().__init__(settings, **kwargs)
         self.debug = False
 
         self.abort = False
@@ -49,14 +59,26 @@ class PlotPlanner:
 
         self.queue = []
 
-        self.single = Single(self)
-        self.smooth = Smooth(self)
-        self.ppi = PPI(self)
-        self.shift = Shift(self)
-        self.group = Group(self)
+        self.single = None
+        self.smooth = None
+        self.ppi = None
+        self.shift = None
+        self.group = None
+
+        if single:
+            self.single = Single(self)
+        if smooth:
+            self.smooth = Smooth(self)
+        if ppi:
+            self.ppi = PPI(self)
+        if shift:
+            self.shift = Shift(self)
+        if group:
+            self.group = Group(self)
 
         self.pos_x = None
         self.pos_y = None
+        self.settings_then_jog = False
 
     def push(self, plot):
         self.abort = False
@@ -81,7 +103,7 @@ class PlotPlanner:
         * Flush steps from planner if going to new location or using new settings.
              * If new position is far
                   * Send jog if too far away.
-        * Send PLOT_SETTING: None None - if settings have changed.
+        * Send PLOT_SETTING: None, None - if settings have changed.
         * Send PLOT_AXIS: major_axis, None - Major axis is horizontal vs. vertical raster
         * Send PLOT_DIRECTION: x_dir(), y_dir() - Direction X, Direction Y for initial cut.
         * Send PLOT_LEFT_UPPER: left point, upper point - Point at upper left
@@ -89,36 +111,38 @@ class PlotPlanner:
         * Send all X, Y points for cut.
 
         If the next position is too far away and jogging is allowed we jog to the position. This
-        jog is sent after the previous data is planner is
+        jog is sent after the previous data is flushed out of the planner.
 
-        :return:
+        cut.settings can be the same object between different cut.
+
+        @return:
         """
         self.pos_x = None
         self.pos_y = None
         while len(self.queue):
             cut = self.queue.pop(0)
-            cut_set = cut.settings
-            cur_set = self.settings
-            start = cut.start()
-            new_start_x = int(start.x)
-            new_start_y = int(start.y)
+            start = cut.start
+            new_start_x = start[0]
+            new_start_y = start[1]
+            assert isinstance(new_start_x, int)
+            assert isinstance(new_start_y, int)
             flush = False
             jog = 0
             if self.pos_x != new_start_x or self.pos_y != new_start_y:
                 # This location is disjointed. We must flush and jog.
                 # Jog is executed in current settings.
-                if self.pos_x is None or cur_set.raster_step != 0:
+                if self.pos_x is None or self.raster_step_x != 0:
                     # First movement or raster_step exists we must rapid_jog.
                     # Request rapid move new location
                     flush = True
                     jog |= PLOT_RAPID
                 else:
                     # Slow Jog to position.
-                    distance = cur_set.jog_distance
+                    distance = self.jog_distance
                     if (
                         abs(self.pos_x - new_start_x) < distance
                         and abs(self.pos_y - new_start_y) < distance
-                    ) or not cur_set.jog_enable:
+                    ) or not self.jog_enable:
                         # Jog distance smaller than threshold. Or jog isn't allowed
                         def walk():
                             for event in ZinglPlotter.plot_line(
@@ -133,7 +157,7 @@ class PlotPlanner:
                         jog |= PLOT_JOG
 
             # Laser Setting has changed, we must flush the buffer.
-            if cut_set is not cur_set:
+            if cut.settings is not self.settings:
                 flush = True
 
             # Flush executed in current settings.
@@ -143,15 +167,24 @@ class PlotPlanner:
                 self.pos_y = self.single.single_y
 
             # Jog was needed.
-            if jog:
-                yield new_start_x, new_start_y, jog
-                self.warp(new_start_x, new_start_y)
+            if self.settings_then_jog:
+                if cut.settings is not self.settings:
+                    self.settings = cut.settings
+                    yield None, None, PLOT_SETTING
 
-            self.settings = cut.settings
-            if cut_set is not cur_set:
-                yield None, None, PLOT_SETTING
+                if jog:
+                    yield new_start_x, new_start_y, jog
+                    self.warp(new_start_x, new_start_y)
+            else:
+                if jog:
+                    yield new_start_x, new_start_y, jog
+                    self.warp(new_start_x, new_start_y)
 
-            if jog or cut_set.raster_step != 0:
+                if cut.settings is not self.settings:
+                    self.settings = cut.settings
+                    yield None, None, PLOT_SETTING
+
+            if jog or self.raster_step_x != 0:
                 # set the directions. Post Jog, Post Settings.
                 yield cut.major_axis(), None, PLOT_AXIS
                 yield cut.x_dir(), cut.y_dir(), PLOT_DIRECTION
@@ -181,48 +214,39 @@ class PlotPlanner:
         Processes can buffer data and return None. Processes are required to surrender any buffer they have if the
         given sequence ends with, or is None. This flushes out any data.
 
-        If an input sequence still lacks a on-value then the single_default value will be utilized.
+        If an input sequence still lacks an on-value then the single_default value will be utilized.
         Output sequences are iterables of x, y, on positions.
 
-        :param plot: plottable element that should be wrapped
-        :return: generator to produce plottable elements.
+        @param plot: plottable element that should be wrapped
+        @return: generator to produce plottable elements.
         """
-        # Applies single, ppi, shift, then group.
+
+        def debug(plot, manipulator):
+            for q in plot:
+                print(f"Manipulator: {str(q)}, {str(manipulator)}")
+                yield q
+
+        if self.single is not None:
+            plot = self.single.process(plot)
         if self.debug:
-
-            def debug(plot, manipulator):
-                for q in plot:
-                    print("Manipulator: %s, %s" % (str(q), str(manipulator)))
-                    yield q
-
-            return debug(
-                self.group.process(
-                    debug(
-                        self.shift.process(
-                            debug(
-                                self.ppi.process(
-                                    debug(
-                                        self.smooth.process(
-                                            debug(
-                                                self.single.process(plot), self.single
-                                            )
-                                        ),
-                                        self.smooth,
-                                    )
-                                ),
-                                self.ppi,
-                            )
-                        ),
-                        self.shift,
-                    )
-                ),
-                self.group,
-            )
-        return self.group.process(
-            self.shift.process(
-                self.ppi.process(self.smooth.process(self.single.process(plot)))
-            )
-        )
+            plot = debug(plot, self.single)
+        if self.smooth is not None:
+            plot = self.smooth.process(plot)
+        if self.debug:
+            plot = debug(plot, self.smooth)
+        if self.ppi is not None:
+            plot = self.ppi.process(plot)
+        if self.debug:
+            plot = debug(plot, self.ppi)
+        if self.shift is not None:
+            plot = self.shift.process(plot)
+        if self.debug:
+            plot = debug(plot, self.shift)
+        if self.group is not None:
+            plot = self.group.process(plot)
+        if self.debug:
+            plot = debug(plot, self.group)
+        return plot
 
     def step_move(self, x0, y0, x1, y1):
         """
@@ -241,18 +265,28 @@ class PlotPlanner:
     def warp(self, x, y):
         self.pos_x = x
         self.pos_y = y
-        self.single.warp(x, y)
-        self.smooth.warp(x, y)
-        self.ppi.warp(x, y)
-        self.shift.warp(x, y)
-        self.group.warp(x, y)
+        if self.single:
+            self.single.warp(x, y)
+        if self.smooth:
+            self.smooth.warp(x, y)
+        if self.ppi:
+            self.ppi.warp(x, y)
+        if self.shift:
+            self.shift.warp(x, y)
+        if self.group:
+            self.group.warp(x, y)
 
     def reset(self):
-        self.single.clear()
-        self.smooth.clear()
-        self.shift.clear()
-        self.ppi.clear()
-        self.group.clear()
+        if self.single:
+            self.single.clear()
+        if self.smooth:
+            self.smooth.clear()
+        if self.shift:
+            self.shift.clear()
+        if self.ppi:
+            self.ppi.clear()
+        if self.group:
+            self.group.clear()
 
 
 class PlotManipulation:
@@ -283,11 +317,7 @@ class Single(PlotManipulation):
         self.single_y = None
 
     def __str__(self):
-        return "%s(%s,%s)" % (
-            self.__class__.__name__,
-            str(self.single_x),
-            str(self.single_y),
-        )
+        return f"{self.__class__.__name__}({str(self.single_x)},{str(self.single_y)})"
 
     def process(self, plot):
         """
@@ -299,8 +329,8 @@ class Single(PlotManipulation):
         single_x sets the last known x position this routine has encountered.
         single_y sets the last known y position this routine has encountered.
 
-        :param plot: plot generator
-        :return:
+        @param plot: plot generator
+        @return:
         """
         if plot is None:
             # None calls for a flush routine.
@@ -325,8 +355,7 @@ class Single(PlotManipulation):
             if total_dy * dx != total_dx * dy:
                 # Check for cross-equality.
                 raise ValueError(
-                    "Must be uniformly diagonal or orthogonal: (%d, %d) is not."
-                    % (total_dx, total_dy)
+                    f"Must be uniformly diagonal or orthogonal: ({total_dx}, {total_dy}) is not."
                 )
             cx = self.single_x
             cy = self.single_y
@@ -362,11 +391,7 @@ class Smooth(PlotManipulation):
         self.smooth_y = None
 
     def __str__(self):
-        return "%s(%s,%s)" % (
-            self.__class__.__name__,
-            str(self.smooth_x),
-            str(self.smooth_y),
-        )
+        return f"{self.__class__.__name__}({str(self.smooth_x)},{str(self.smooth_y)})"
 
     def flushed(self):
         return (
@@ -378,9 +403,8 @@ class Smooth(PlotManipulation):
 
     def process(self, plot):
         """
-
-        :param plot: single stepped plots to be smoothed into orth/diag sequences.
-        :return:
+        @param plot: Smooth attempts to smooth out values
+        @return:
         """
         px = None
         py = None
@@ -390,10 +414,9 @@ class Smooth(PlotManipulation):
                 yield from self.flush()
                 yield x, y, on
                 continue
-            if (
-                not self.planner.settings.constant_move_x
-                and not self.planner.settings.constant_move_y
-            ):
+            if not self.planner.settings.get(
+                "_constant_move_x", False
+            ) and not self.planner.settings.get("_constant_move_y", False):
                 yield x, y, on
                 continue  # We are not smoothing.
             if px is not None and py is not None:
@@ -411,14 +434,15 @@ class Smooth(PlotManipulation):
             self.goal_y = y
             self.goal_on = on
             if total_dx == 0 and total_dy == 0:
+                yield x, y, on
                 continue
             dx = 1 if total_dx > 0 else 0 if total_dx == 0 else -1
             dy = 1 if total_dy > 0 else 0 if total_dy == 0 else -1
-            if self.planner.settings.constant_move_x and dx == 0:
-                # If we are moving x and we don't move x. Skip.
+            if self.planner.settings.get("_constant_move_x", False) and dx == 0:
+                # If we are moving x and, we don't move x: skip.
                 if abs(total_dy) < self.planner.smooth_limit:
                     continue
-            if self.planner.settings.constant_move_y and dy == 0:
+            if self.planner.settings.get("_constant_move_y", False) and dy == 0:
                 if abs(total_dx) < self.planner.smooth_limit:
                     continue
             self.smooth_x += dx
@@ -435,6 +459,8 @@ class Smooth(PlotManipulation):
                 self.smooth_x, self.smooth_y, self.goal_x, self.goal_y
             ):
                 yield x, y, self.goal_on
+            self.smooth_x = None
+            self.smooth_y = None
             self.goal_x = None
             self.goal_y = None
             self.goal_on = None
@@ -457,11 +483,7 @@ class PPI(PlotManipulation):
         self.dot_left = 0
 
     def __str__(self):
-        return "%s(%s,%s)" % (
-            self.__class__.__name__,
-            str(self.ppi_total),
-            str(self.dot_left),
-        )
+        return f"{self.__class__.__name__}({str(self.ppi_total)},{str(self.dot_left)})"
 
     def process(self, plot):
         """
@@ -469,8 +491,8 @@ class PPI(PlotManipulation):
 
         Implements PPI power modulation.
 
-        :param plot: generator of single stepped plots, with PPI.
-        :return:
+        @param plot: generator of single stepped plots, with PPI.
+        @return:
         """
         px = None
         py = None
@@ -483,7 +505,7 @@ class PPI(PlotManipulation):
             px = x
             py = y
             # PPI is always on.
-            self.ppi_total += self.planner.settings.power * on
+            self.ppi_total += self.planner.power * on
             if on and self.dot_left > 0:
                 # Process remaining dot_length, must be on or partially on.
                 self.dot_left -= 1
@@ -493,8 +515,8 @@ class PPI(PlotManipulation):
                 if self.ppi_total >= 1000.0:
                     # PPI >= 1000: triggers on.
                     on = 1
-                    self.ppi_total -= 1000.0 * self.planner.settings.dot_length
-                    self.dot_left = self.planner.settings.dot_length - 1
+                    self.ppi_total -= 1000.0 * self.planner.dot_length
+                    self.dot_left = self.planner.dot_length - 1
                 else:
                     # PPI < 1000: triggers off.
                     on = 0
@@ -508,11 +530,7 @@ class Shift(PlotManipulation):
         self.shift_pixels = 0
 
     def __str__(self):
-        return "%s(%s,%s)" % (
-            self.__class__.__name__,
-            str(self.shift_buffer),
-            bin(self.shift_pixels),
-        )
+        return f"{self.__class__.__name__}({str(self.shift_buffer)},{bin(self.shift_pixels)})"
 
     def process(self, plot):
         """
@@ -520,12 +538,12 @@ class Shift(PlotManipulation):
 
         This code requires a buffer of 4 path plots.
 
-        :param plot: generator of single stepped plots
-        :return:
+        @param plot: generator of single stepped plots
+        @return:
         """
         for x, y, on in plot:
             if (x is None or y is None) or (
-                not self.planner.force_shift and not self.planner.settings.shift_enabled
+                not self.planner.force_shift and not self.planner.shift_enabled
             ):
                 # If we have an established buffer, flush the buffer.
                 yield from self.flush()
@@ -586,13 +604,7 @@ class Group(PlotManipulation):
         self.group_dy = 0
 
     def __str__(self):
-        return "%s(%s,%s,%s,%s)" % (
-            self.__class__.__name__,
-            str(self.group_x),
-            str(self.group_y),
-            str(self.group_dx),
-            str(self.group_dy),
-        )
+        return f"{self.__class__.__name__}({str(self.group_x)},{str(self.group_y)},{str(self.group_dx)},{str(self.group_dy)})"
 
     def flushed(self):
         return (
@@ -608,8 +620,8 @@ class Group(PlotManipulation):
         group_x, group_y: is the last known inputted. This is not necessarily the current position given by the planner.
         If group_dx and group_dy are not zero, then we have buffered a move.
 
-        :param plot: single stepped plots to be grouped into orth/diag sequences.
-        :return:
+        @param plot: single stepped plots to be grouped into orth/diag sequences.
+        @return:
         """
         px = None
         py = None
@@ -658,7 +670,7 @@ class Group(PlotManipulation):
             if abs(self.group_dx) > 1 or abs(self.group_dy) > 1:
                 # The last step was not valid. Group() requires single step values.
                 raise ValueError(
-                    "dx(%d) or dy(%d) exceeds 1" % (self.group_dx, self.group_dy)
+                    f"dx({self.group_dx}) or dy({self.group_dy}) exceeds 1"
                 )
             # Save our buffered position.
             self.group_x = x
@@ -696,8 +708,8 @@ def grouped(plot):
     """
     Converts a generated series of single stepped plots into grouped orthogonal/diagonal plots.
 
-    :param plot: single stepped plots to be grouped into orth/diag sequences.
-    :return:
+    @param plot: single stepped plots to be grouped into orth/diag sequences.
+    @return:
     """
     group_x = None
     group_y = None
@@ -721,7 +733,7 @@ def grouped(plot):
         group_dy = y - group_y
         if abs(group_dx) > 1 or abs(group_dy) > 1:
             # The last step was not valid.
-            raise ValueError("dx(%d) or dy(%d) exceeds 1" % (group_dx, group_dy))
+            raise ValueError(f"dx({group_dx}) or dy({group_dy}) exceeds 1")
         group_x = x
         group_y = y
     # There are no more plots.
