@@ -36,7 +36,7 @@ def convert_to_list_bytes(data):
 class Ch341LibusbDriver:
     """
     Libusb driver for the CH341 chip. The CH341x is a USB interface chip that can emulate UART, parallel port,
-    and synchronous serial (EPP, I2C, SPI). The Lhystudios boards (M2Nano, et al) and Moshiboards and likely others
+    and synchronous serial (EPP, I2C, SPI). The Lihuiyu boards (M2Nano, et al.) and Moshiboards and likely others
     use this in EPP 1.9 mode. This class is not and not intended to be a full driver for that chip. Rather we are
     duplicating the function calls and implementing compatible operations to permit a stand-in for the default CH341
     driver. Using libusb to do the usb connection to the chip as well as permitting easy swapping that for the default
@@ -109,6 +109,7 @@ class Ch341LibusbDriver:
         _ = self.channel._
         try:
             if device.is_kernel_driver_active(interface.bInterfaceNumber):
+                # TODO: This can raise USBError on entity not found.
                 try:
                     self.channel(_("Attempting to detach kernel."))
                     device.detach_kernel_driver(interface.bInterfaceNumber)
@@ -350,7 +351,7 @@ class Ch341LibusbDriver:
 
     # pylint: disable=dangerous-default-value
     def CH341GetStatus(self, index=0, status=[0]):
-        """D7-0, 8: err, 9: pEmp, 10: Int, 11: SLCT, 12: SDA, 13: Busy, 14: datas, 15: addrs"""
+        """D7-0, 8: err, 9: pEmp, 10: Int, 11: SLCT, 12: SDA, 13: Busy, 14: data, 15: addrs"""
         device = self.devices[index]
         try:
             device.write(
