@@ -1,9 +1,7 @@
-from meerk40t.core.elements import MILS_IN_MM
-from meerk40t.svgelements import Angle, Length, Matrix, Path, Polyline, Shape
+from meerk40t.core.units import Length
+from meerk40t.svgelements import Angle, Matrix, Path, Polyline
 from meerk40t.tools.pathtools import EulerianFill
 
-MILS_PER_MM = 39.3701
-
 
 def plugin(kernel, lifecycle):
     if lifecycle == "register":
@@ -11,31 +9,31 @@ def plugin(kernel, lifecycle):
         context = kernel.root
 
         @context.console_option(
-            "angle", "a", type=Angle.parse, default=0, help=_("Angle of the fill")
+            "angle", "a", type=Angle.parse, default="0deg", help=_("Angle of the fill")
         )
         @context.console_option(
-            "distance", "d", type=Length, default=16, help=_("Length between rungs")
+            "distance",
+            "d",
+            type=Length,
+            default="0.5mm",
+            help=_("Length between rungs"),
         )
         @context.console_command("embroider", help=_("embroider <angle> <distance>"))
         def embroider(command, channel, _, angle=None, distance=None, **kwargs):
-            bed_dim = context.root
             elements = context.elements
             channel(_("Embroidery Filling"))
-            if distance is not None:
-                distance = distance.value(
-                    ppi=1000.0, relative_length=bed_dim.bed_height * MILS_IN_MM
-                )
-            else:
-                distance = 16
-
-            efill = EulerianFill(distance)
-            for element in elements.elems(emphasized=True):
-                if not isinstance(element, Shape):
-                    continue
-                e = Path(element)
+            efill = EulerianFill(float(distance))
+            for node in elements.elems(emphasized=True):
+                try:
+                    path = Path(node.shape)
+                except AttributeError:
+                    try:
+                        path = Path(node.path)
+                    except AttributeError:
+                        continue
                 if angle is not None:
-                    e *= Matrix.rotate(angle)
-                pts = [abs(e).point(i / 100.0, error=1e-4) for i in range(101)]
+                    path *= Matrix.rotate(angle)
+                pts = [abs(path).point(i / 100.0, error=1e-4) for i in range(101)]
                 efill += pts
 
             points = efill.get_fill()
@@ -44,7 +42,8 @@ def plugin(kernel, lifecycle):
                 result = Path(Polyline(s, stroke="black"))
                 if angle is not None:
                     result *= Matrix.rotate(-angle)
-                elements.add_elem(result)
+                node = elements.elem_branch.add(path=result, type="elem path")
+                elements.classify([node])
 
 
 def split(points):
