@@ -12,6 +12,7 @@ class GraphNode(Point):
         Point.__init__(self, x, y)
         self.connections = []
         self.visited = 0
+        self.value = None
 
 
 class Segment:
@@ -36,12 +37,7 @@ class Segment:
         return 9
 
     def __str__(self):
-        return "Segment(%s,%s,%s,type='%s')" % (
-            str(self.a),
-            str(self.b),
-            str(self.index),
-            self.value,
-        )
+        return f"Segment({str(self.a)},{str(self.b)},{str(self.index)},type='{self.value}')"
 
     def __getitem__(self, item):
         if item == 0:
@@ -116,7 +112,7 @@ class Graph:
     A graph is a set of nodes and their connections. The nodes are points within 2d space and any number of segments
     can connect any number of points. There is no order established by the graph. And for our uses here all graphs will
     end up not only being Eulerian but Euloopian. All nodes should have even numbers of connecting segments so that any
-    walk will always return back to the end location.
+    walk will always return to the end location.
 
     """
 
@@ -239,7 +235,7 @@ class Graph:
         Makes any graph Eulerian. Any graph that is doubled is by definition Eulerian.
 
         This is not used by the algorithm.
-        :return:
+        @return:
         """
         for i in range(len(self.links)):
             s = self.links[i]
@@ -252,11 +248,11 @@ class Graph:
 
     def double_odd_edge(self):
         """
-        Makes any outline path a Eularian path, by doubling every other edge. As each node connects with 1 rung, and
+        Makes any outline path an Eularian path, by doubling every other edge. As each node connects with 1 rung, and
         two edges this will double 1 of those edges in every instance, giving a total of 4 connections. This is makes
         the graph Eulerian.
 
-        :return:
+        @return:
         """
         for i in range(len(self.links)):
             segment = self.links[i]
@@ -267,7 +263,7 @@ class Graph:
 
     def walk(self, points):
         """
-        We have a Eulerian graph we must walk through the graph in any direction. This results in a point series that
+        We have an Eulerian graph we must walk through the graph in any direction. This results in a point series that
         will cross every segment once.
 
         Some segments are marked scaffolding or classes of edge that are not necessary. These are removed for parsimony.
@@ -341,12 +337,12 @@ class GraphWalker:
 
     def make_walk_node(self, g):
         """
-        Starting from the given start node it makes a complete walk in a Eulerian circuit.
+        Starting from the given start node it makes a complete walk in an Eulerian circuit.
 
         It adds the first loop from the start node, then walks its looped walk adding
         any additional loops it finds to the current loop.
-        :param g:
-        :return:
+        @param g:
+        @return:
         """
         start = len(self.walk)
         self.walk.append(g)
@@ -370,9 +366,9 @@ class GraphWalker:
         Travels along unused connections until no more travel is possible. If properly Eulerian,
         this will only happen when it is looped back on itself.
 
-        :param index: index we are adding loop to.
-        :param node: Node to find alternative path through.
-        :return: new index after loop is added to the walk.
+        @param index: index we are adding loop to.
+        @param node: Node to find alternative path through.
+        @return: new index after loop is added to the walk.
         """
         index += 1
         i = index
@@ -394,8 +390,8 @@ class GraphWalker:
         """
         Finds the first unused edge segment within the graph node, or None if all connections are used.
 
-        :param node: Node to find unused edge segment within.
-        :return: index of node connection within the graphnode
+        @param node: Node to find unused edge segment within.
+        @return: index of node connection within the graphnode
         """
         value = None
         for index, c in enumerate(node.connections):
@@ -410,22 +406,29 @@ class GraphWalker:
         """
         Adds nodes within the walk to the points given to it.
 
-        :param points:
-        :return:
+        @param points:
+        @return:
         """
         for i in range(0, len(self.walk), 2):
-            if self.walk[i - 1] is None:
+            segment = self.walk[i - 1]
+            # The first time segment will be the last value (a node) which will set value to none. This is fine.
+            point = self.walk[i]
+            if segment is None:
                 points.append(None)
-            points.append(self.walk[i])
+            else:
+                point.value = (
+                    segment.value
+                )  # This doesn't work, nodes are repeated, so they can't store unique values.
+            points.append(point)
 
     def remove_loop(self, from_pos, to_pos):
         """
         Removes values between the two given points.
         Since start and end are the same node, it leaves one in place.
 
-        :param from_pos:
-        :param to_pos:
-        :return:
+        @param from_pos:
+        @param to_pos:
+        @return:
         """
         if from_pos == to_pos:
             return 0
@@ -442,9 +445,9 @@ class GraphWalker:
         It iterates from the outside to the center, setting the visited value for each node.
 
         If it finds a marked node, that is the biggest loop within the given walk.
-        :param start:
-        :param end:
-        :return:
+        @param start:
+        @param end:
+        @return:
         """
         for i in range(start, end + 2, 2):
             n = self.get_node(i)
@@ -472,13 +475,12 @@ class GraphWalker:
 
         Clips unneeded scaffolding.
 
-        :return:
+        @return:
         """
         start = 0
         index = 0
         ie = len(self.walk)
         while index < ie:
-            segment = None
             try:
                 segment = self.walk[index + 1]
             except IndexError:
@@ -492,7 +494,6 @@ class GraphWalker:
             index += 2
 
     def remove_scaffold_ends_in_range(self, start, end):
-        current = end - start
         new_end = end
         limit = start + 2
         while new_end >= limit:
@@ -512,7 +513,6 @@ class GraphWalker:
                 if new_start == start:
                     break
                 del self.walk[start:new_start]
-                start = new_start
                 break
             new_start += 2
 
@@ -521,7 +521,6 @@ class GraphWalker:
         end = len(self.walk) - 1
         index = end
         while index >= 0:
-            segment = None
             try:
                 segment = self.walk[index - 1]
             except IndexError:
@@ -605,7 +604,7 @@ class GraphWalker:
     def get_value(self):
         """
         Path values with flip.
-        :return: Flipped path value.
+        @return: Flipped path value.
         """
         if len(self.walk) == 0:
             return 0
@@ -642,7 +641,6 @@ class GraphWalker:
                         # Only skippable nodes existed before returned to original node, so skip that loop.
                         value += (k - j) * 10
                         j = k
-                        j_node = k_node
                         j_segment = k_segment
                         break
             if j_segment.value == "SCAFFOLD":
@@ -700,12 +698,12 @@ class VectorMontonizer:
             else:
                 high = p1
                 low = p0
-            try:
-                m = (high.y - low.y) / (high.x - low.x)
-            except ZeroDivisionError:
-                m = float("inf")
+            # try:
+            #     m = (high.y - low.y) / (high.x - low.x)
+            # except ZeroDivisionError:
+            #     m = float("inf")
 
-            b = low.y - (m * low.x)
+            # b = low.y - (m * low.x)
             if self.valid_low_value > high.y:
                 continue  # Cluster before range.
             if self.valid_high_value < low.y:
@@ -842,6 +840,19 @@ class EulerianFill:
         self.outlines.append(other)
         return self
 
+    def estimate(self):
+        min_y = float("inf")
+        max_y = -float("inf")
+        for outline in self.outlines:
+            o_min_y = min([p[1] for p in outline])
+            o_max_y = max([p[1] for p in outline])
+            min_y = min(min_y, o_min_y)
+            max_y = max(max_y, o_max_y)
+        try:
+            return (max_y - min_y) / self.distance
+        except ZeroDivisionError:
+            return float("inf")
+
     def get_fill(self):
         min_y = float("inf")
         max_y = -float("inf")
