@@ -3,11 +3,14 @@ import unittest
 
 from PIL import Image, ImageDraw
 
-from meerk40t.core.cutcode import CutCode, LaserSettings, LineCut
-from meerk40t.core.elements import LaserOperation
+from meerk40t.core.cutcode import CutCode, LineCut
+from meerk40t.core.node.elem_image import ImageNode
+from meerk40t.core.node.elem_path import PathNode
+from meerk40t.core.node.op_engrave import EngraveOpNode
+from meerk40t.core.node.op_raster import RasterOpNode
 from meerk40t.core.plotplanner import PlotPlanner
 from meerk40t.device.basedevice import PLOT_AXIS, PLOT_SETTING
-from meerk40t.svgelements import Circle, Path, Point, SVGImage
+from meerk40t.svgelements import Circle, Matrix, Path, Point
 
 
 class TestPlotplanner(unittest.TestCase):
@@ -16,9 +19,8 @@ class TestPlotplanner(unittest.TestCase):
         With raster_smooth set to 1 we should smooth the x axis so that no y=0 occurs.
         @return:
         """
-        plan = PlotPlanner(LaserSettings(power=1000))
-        settings = LaserSettings(power=1000)
-        settings.constant_move_x = True
+        settings = {"power": 1000, "_constant_move_x": True}
+        plan = PlotPlanner(settings, ppi=False)
         plan.push(LineCut(Point(0, 0), Point(20, 2), settings=settings))
         plan.push(LineCut(Point(20, 2), Point(20, 5), settings=settings))
         plan.push(LineCut(Point(20, 5), Point(100, 10), settings=settings))
@@ -54,10 +56,8 @@ class TestPlotplanner(unittest.TestCase):
         With raster_smooth set to 1 we should smooth the x axis so that no y=0 occurs.
         @return:
         """
-        plan = PlotPlanner(LaserSettings(power=1000))
-        settings = LaserSettings(power=1000)
-        settings.constant_move_x = True
-        settings.power = 500
+        settings = {"power": 500, "_constant_move_x": True}
+        plan = PlotPlanner(settings)
         plan.push(LineCut(Point(0, 0), Point(20, 2), settings=settings))
         plan.push(LineCut(Point(20, 2), Point(20, 5), settings=settings))
         plan.push(LineCut(Point(20, 5), Point(100, 10), settings=settings))
@@ -97,9 +97,8 @@ class TestPlotplanner(unittest.TestCase):
         With smooth_raster set to 2 we should never have x = 0. The x should *always* be in motion.
         @return:
         """
-        plan = PlotPlanner(LaserSettings(power=1000))
-        settings = LaserSettings(power=1000)
-        settings.constant_move_y = True
+        settings = {"power": 1000, "_constant_move_y": True}
+        plan = PlotPlanner(settings)
         plan.push(LineCut(Point(0, 0), Point(2, 20), settings=settings))
         plan.push(LineCut(Point(2, 20), Point(5, 20), settings=settings))
         plan.push(LineCut(Point(5, 20), Point(10, 100), settings=settings))
@@ -135,10 +134,10 @@ class TestPlotplanner(unittest.TestCase):
         With raster_smooth set to 1 we should smooth the x axis so that no y=0 occurs.
         @return:
         """
-        plan = PlotPlanner(LaserSettings(power=1000))
-        settings = LaserSettings(power=1000)
-        settings.constant_move_x = True
-        settings.constant_move_y = True
+        settings = {"power": 1000, "_constant_move_x": True, "_constant_move_y": True}
+        plan = PlotPlanner(settings)
+        self.constant_move_x = True
+        self.constant_move_y = True
         for i in range(100):
             plan.push(
                 LineCut(
@@ -179,18 +178,26 @@ class TestPlotplanner(unittest.TestCase):
         @return:
         """
         for q in range(100):
-            plan = PlotPlanner(LaserSettings(power=1000))
-            settings = LaserSettings(power=1000)
-            settings.constant_move_x = bool(random.randint(0, 1))
-            settings.constant_move_y = bool(random.randint(0, 1))
+            settings = {
+                "power": 1000,
+                "constant_move_x": bool(random.randint(0, 1)),
+                "constant_move_y": bool(random.randint(0, 1)),
+            }
+            plan = PlotPlanner(settings)
             goal_x = None
             goal_y = None
             for i in range(10):
                 goal_x = random.randint(0, 100)
                 goal_y = random.randint(0, 100)
+                start_x = random.randint(0, 100)
+                start_y = random.randint(0, 100)
+                while start_x == goal_x and start_y == goal_y:
+                    # If exactly equal go ahead and randomize again.
+                    start_x = random.randint(0, 100)
+                    start_y = random.randint(0, 100)
                 plan.push(
                     LineCut(
-                        Point(random.randint(0, 100), random.randint(0, 100)),
+                        Point(start_x, start_y),
                         Point(goal_x, goal_y),
                         settings=settings,
                     )
@@ -208,24 +215,26 @@ class TestPlotplanner(unittest.TestCase):
                 last_y = y
 
             if last_x != goal_x:
-                print(settings.constant_move_x)
-                print(settings.constant_move_y)
+                print(settings.get("constant_move_x"))
+                print(settings.get("constant_move_y"))
                 for seg in break_list:
                     print(repr(seg))
             self.assertEqual(last_x, goal_x)
             if last_y != goal_y:
-                print(settings.constant_move_x)
-                print(settings.constant_move_y)
+                print(settings.get("constant_move_x"))
+                print(settings.get("constant_move_y"))
                 for seg in break_list:
                     print(repr(seg))
             self.assertEqual(last_y, goal_y)
 
     def test_plotplanner_static_issue(self):
-        plan = PlotPlanner(LaserSettings(power=1000))
+        settings = {
+            "power": 1000,
+            "constant_move_x": True,
+            "constant_move_y": False,
+        }
+        plan = PlotPlanner(settings)
         plan.debug = True
-        settings = LaserSettings(power=1000)
-        settings.constant_move_x = True
-        settings.constant_move_y = False
         lines = (
             ((41, 45), (14, 43)),
             ((32, 67), (32, 61)),
@@ -266,11 +275,8 @@ class TestPlotplanner(unittest.TestCase):
         With raster_smooth set to 1 we should smooth the x axis so that no y=0 occurs.
         @return:
         """
-        plan = PlotPlanner(LaserSettings(power=1000))
-        # plan.debug = True
-        settings = LaserSettings(power=1000)
-        settings.constant_move_x = True
-        settings.constant_move_y = True
+        settings = {"power": 1000, "constant_move_x": True, "constant_move_y": True}
+        plan = PlotPlanner(settings)
         plan.push(LineCut(Point(0, 0), Point(0, 100), settings=settings))
         plan.push(LineCut(Point(0, 100), Point(100, 100), settings=settings))
         plan.push(LineCut(Point(100, 100), Point(100, 0), settings=settings))
@@ -317,16 +323,13 @@ class TestPlotplanner(unittest.TestCase):
 
         :return:
         """
-        plan = PlotPlanner(LaserSettings(power=1000))
-        settings = LaserSettings(power=1000)
+        settings = {"power": 1000}
+        plan = PlotPlanner(settings)
+
         for i in range(211):
             plan.push(LineCut(Point(0, 0), Point(5, 100), settings=settings))
             plan.push(LineCut(Point(100, 50), Point(0, 0), settings=settings))
-            plan.push(
-                LineCut(
-                    Point(50, -50), Point(100, -100), settings=LaserSettings(power=0)
-                )
-            )
+            plan.push(LineCut(Point(50, -50), Point(100, -100), settings={"power": 0}))
             q = 0
             for x, y, on in plan.gen():
                 # print(x, y, on)
@@ -356,20 +359,28 @@ class TestPlotplanner(unittest.TestCase):
         :return:
         """
 
-        rasterop = LaserOperation(operation="Raster")
-        svg_image = SVGImage()
-        svg_image.image = Image.new("RGBA", (256, 256))
-        draw = ImageDraw.Draw(svg_image.image)
+        rasterop = RasterOpNode()
+        image = Image.new("RGBA", (256, 256))
+        draw = ImageDraw.Draw(image)
         draw.ellipse((0, 0, 255, 255), "black")
-        rasterop.add(svg_image, type="opnode")
+        image = image.convert("L")
+        inode = ImageNode(image=image, dpi=1000.0, matrix=Matrix())
+        inode.step_x = 1
+        inode.step_y = 1
+        inode.process_image()
+        rasterop.add_node(inode)
+        rasterop.raster_step_x = 1
+        rasterop.raster_step_y = 1
 
-        vectorop = LaserOperation(operation="Engrave")
-        vectorop.add(Path(Circle(cx=127, cy=127, r=128, fill="black")), type="opnode")
+        vectorop = EngraveOpNode()
+        vectorop.add_node(
+            PathNode(path=Path(Circle(cx=127, cy=127, r=128)), fill="black")
+        )
         cutcode = CutCode()
         cutcode.extend(vectorop.as_cutobjects())
         cutcode.extend(rasterop.as_cutobjects())
-
-        plan = PlotPlanner(LaserSettings(power=500))
+        settings = {"power": 500}
+        plan = PlotPlanner(settings)
         for c in cutcode.flat():
             plan.push(c)
 
